'use strict';

const _ = require('lodash');
const Promise = require('bluebird');
const YAML = require('js-yaml');
const Base = require('./../../kube-native/resource.base.controller');
const ReleasePool = require('./pool');
const { parseFamiliarName } = require('@codefresh-io/docker-reference');

const TILLER_LABEL='OWNER=TILLER';

class ReleaseController extends Base {

    _getRelevantConfigmaps(name){
        const basicLabel = TILLER_LABEL;
        return this._kubeManager
            .getConfigmapController(this._ns)
            .get({labelSelector: name ? `${name},${basicLabel}` : basicLabel});
    }

    _ensureTillerExistOnNamespace(tillerServiceName = 'tiller-deploy'){
        return this._kubeManager
            .getServiceController(this._ns)
            .describe(tillerServiceName)
            .catch((err) => {
                throw new Error(`Tiller not found in namespace ${this._ns}`);
            });
    }

    get() {
        return this._ensureTillerExistOnNamespace()
        .then(() => this._getRelevantConfigmaps())
        .then((configmaps) => {
            const pool = new ReleasePool({ configmaps });
            return pool.prepareReleases()   
                .then(() => pool.getAllReleases());
        });
    }

    describe(name) {
        let finalRelease;
        return this._ensureTillerExistOnNamespace()
        .then(() => this._getRelevantConfigmaps(`NAME=${name}`))
        .then((configmaps) => {
            if(!configmaps || _.isEmpty(configmaps)){
                throw new Error(`Release ${name} not found`);
            }
            const pool = new ReleasePool({ configmaps });
            return pool.prepareReleases()
            .then(() => pool.getReleaseFromPool(name))
        });
    }

    create(resourceDescriptor) {
        throw new Error('Not supported yet');
    }

    delete(name) {
        throw new Error('Not supported yet');
    }

    update(name, resourceDescriptor) {
        throw new Error('Not supported yet');
    }

    publishEvent(name, data) {
        this.emit(name, data);
    }

    getTillerVersion(tillerServiceName = 'tiller-deploy') {
        return this._ensureTillerExistOnNamespace(tillerServiceName)
            .then(() => this._kubeManager.getServiceController(this._ns).describe(tillerServiceName))
            .then(service => _.chain(service.getServiceSelector()).map((v, k) => `${k}=${v}`).join(',').value())
            .then(labelSelector => this._kubeManager.getPodController(this._ns).get({
                labelSelector
            }))
            .then((pods) => {
                if(!pods){
                    throw new Error(`Could not find tiller pod`);
                }
                return pods[0].getImages()
            })
            .then(images => parseFamiliarName(images[0].name))
            .then(refernce => refernce.tag);
    }

    compareRevisionsInRelease(releaseName, base, head){
        return this.describe(releaseName)
        .then(release => release.compareRevisions(base, head));
    }

}

module.exports = ReleaseController;